@@ -9,10 +9,12 @@ from matplotlib.lines import Line2D
 import statsmodels.formula.api as smf
 import warnings
 import logging
+import re
 from collections import namedtuple
 
-__version__ = '0.6.0'
-TEAL = ['#004d40', '#00796b', '#009688', '#4db6ac', '#b2dfdb']
+__version__ = '0.7.1'
+DEFAULT_HUE_COLORMAP = 'Dark2'
+DEFAULT_ANNOTATION_COLORMAP = 'brg'
 DEEP_ORANGE = ['#bf360c', '#e64a19', '#ff5722', '#ff8a65', '#ffccbc']
 LINESTYLES = ['-', '--', ':']
 logger = logging.getLogger('time_series_test')
@@ -111,6 +113,68 @@ def find(dm, formula, groups, re_formula=None, winlen=1, split=4,
                                     **kwargs)
 
 
+def lmer_series(dm, formula, winlen=1, fit_kwargs={}, **kwargs):
+    """Performs a sample-by-sample linear-mixed-effects analysis.
+    
+    Parameters
+    ----------
+    dm: DataMatrix
+    formula: str
+    winlen: int, optional
+    fit_kwargs: dict, optional
+    **kwargs: dict, optional
+    
+    Returns
+    -------
+    DataMatrix
+        A DataMatrix with one row per effect, including the intercept, and
+        three series columns with the same depth as the dependent measure
+        specified in the formula:
+        
+        - `est`: the slope
+        - `p`: the p value
+        - `z`: the z value
+        - `se`: the standard error
+    """
+    terms = _terms(formula, **kwargs)
+    # For efficient memory use, we first strip all columns except the relevant
+    # ones from the datamatrix. We also create an ever leaner copy that doesn't
+    # contain the dependent variable, which is a series column. This leaner
+    # copy will be filled in with a single (non-series) column for the dv for
+    # each iteration of the loop.
+    wm = dm[terms]
+    wm_no_dv = dm[terms[1:]]
+    dv = terms[0]
+    depth = dm[dv].depth
+    rm = None
+    for i in range(0, depth, winlen):
+        logger.debug('sample {}'.format(i))
+        with warnings.catch_warnings():
+            warnings.simplefilter('ignore')
+            wm_no_dv[dv] = srs.reduce(
+                srs.window(wm[dv], start=i, end=i+winlen))
+        try:
+            lm = smf.mixedlm(formula, wm_no_dv[dv] != np.nan, **kwargs).fit(
+                **fit_kwargs)
+        except np.linalg.LinAlgError as e:
+            warnings.warn('failed to fit mode: {}'.format(e))
+            continue
+        length = len(lm.model.exog_names)
+        if rm is None:
+            rm = DataMatrix(length=length)
+            rm.effect = lm.model.exog_names
+            rm.p = SeriesColumn(depth=depth)
+            rm.z = SeriesColumn(depth=depth)
+            rm.est = SeriesColumn(depth=depth)
+            rm.se = SeriesColumn(depth=depth)
+        for sample in range(i, min(depth, i + winlen)):
+            rm.p[:, sample] = list(lm.pvalues[:length])
+            rm.z[:, sample] = list(lm.tvalues[:length])
+            rm.est[:, sample] = list(lm.params[:length])
+            rm.se[:, sample] = list(lm.bse[:length])
+    return rm
+
+
 def plot(dm, dv, hue_factor, results=None, linestyle_factor=None, hues=None,
          linestyles=None, alpha_level=.05, annotate_intercept=False,
          annotation_hues=None, annotation_linestyle=':'):
@@ -135,8 +199,9 @@ def plot(dm, dv, hue_factor, results=None, linestyle_factor=None, hues=None,
     linestyle_factor: str, optional
         The name of a regular (non-series) column in `dm` that specifies the
         linestyle of the lines for a two-factor plot.
-    hues: list or None, optional
-        A list of hues to be used as line colors for the first factor.
+    hues: str, list, or None, optional
+        The name of a matplotlib colormap or a list of hues to be used as line
+        colors for the hue factor.
     linestyles: list or None, optional
         A list of linestyles to be used for the second factor.
     alpha_level: float, optional
@@ -145,20 +210,25 @@ def plot(dm, dv, hue_factor, results=None, linestyle_factor=None, hues=None,
     annotate_intercept: bool, optional
         Specifies whether the intercept should also be annotated along with
         the fixed effects.
-    annotation_hues: list or None, optional
-        A list of hues to be used as line color for the annotations.
+    annotation_hues: str, list, or None, optional
+        The name of a matplotlib colormap or a list of hues to be used for the
+        annotations if `results` is provided.
     annotation_linestyle: str, optional
         The linestyle for the annotations.
     """
     if hues is None:
-        hues = TEAL
-    if annotation_hues is None:
-        annotation_hues = DEEP_ORANGE
+        hues = DEFAULT_HUE_COLORMAP
+    if isinstance(hues, str):
+        hues = _colors(hues, dm[hue_factor].count)
     if linestyles is None:
         linestyles = LINESTYLES
     # Plot the annotations
     annotation_elements = []
     if results is not None:
+        if annotation_hues is None:
+            annotation_hues = DEFAULT_ANNOTATION_COLORMAP
+        if isinstance(annotation_hues, str):
+            annotation_hues = _colors(annotation_hues, len(results))
         i = 0
         for effect, result in results.items():
             if effect == 'Intercept' and not annotate_intercept:
@@ -295,7 +365,7 @@ def _lmer_run_localizer(dm, formula, groups, re_formula=None, winlen=1,
     for test_indices, ref_indices in split_indices:
         logger.debug('test size: {}, reference size: {}'.format(
             len(test_indices), len(ref_indices)))
-        lm = _lmer_series(dm[ref_indices], formula, winlen=winlen,
+        lm = lmer_series(dm[ref_indices], formula, winlen=winlen,
                          groups=groups, re_formula=re_formula,
                          fit_kwargs=fit_kwargs, **kwargs)
         if result_dm is None:
@@ -310,9 +380,9 @@ def _lmer_run_localizer(dm, formula, groups, re_formula=None, winlen=1,
 def _lmer_test_localizer(dm, formula, groups, re_formula=None, winlen=1,
                          target_col='__lmer_localizer__', samples_fe=False,
                          samples_re=False, fit_kwargs={}):
-    test_dm = dm[:]
-    dv = formula.split()[0]
-    del test_dm[dv]
+    terms = _terms(formula, groups=groups, re_formula=re_formula)
+    test_dm = dm[terms[1:]]
+    dv = terms[0]
     signal = dm[dv]._seq
     indices = np.array(dm[target_col]._seq, dtype=int)
     results = {}
@@ -351,34 +421,23 @@ def _lmer_test_localizer(dm, formula, groups, re_formula=None, winlen=1,
     return results
 
 
-def _lmer_series(dm, formula, winlen=1, fit_kwargs={}, **kwargs):
+def _split_terms(formula):
+    """Extracts all terms from a formula."""
+    return [term for term in re.split('[ ~+%*]', formula) if term.strip()]
     
-    col = formula.split()[0]
-    depth = dm[col].depth
-    rm = None
-    for i in range(0, depth, winlen):
-        logger.debug('sample {}'.format(i))
-        wm = dm[:]
-        with warnings.catch_warnings():
-            warnings.simplefilter('ignore')
-            wm[col] = srs.reduce(srs.window(wm[col], start=i, end=i+winlen))
-        wm = wm[col] != np.nan
-        try:
-            lm = smf.mixedlm(formula, wm, **kwargs).fit(**fit_kwargs)
-        except np.linalg.LinAlgError as e:
-            warnings.warn('failed to fit mode: {}'.format(e))
-            continue
-        length = len(lm.model.exog_names)
-        if rm is None:
-            rm = DataMatrix(length=length)
-            rm.effect = lm.model.exog_names
-            rm.p = SeriesColumn(depth=depth)
-            rm.z = SeriesColumn(depth=depth)
-            rm.est = SeriesColumn(depth=depth)
-            rm.se = SeriesColumn(depth=depth)
-        for sample in range(i, min(depth, i + winlen)):
-            rm.p[:, sample] = list(lm.pvalues[:length])
-            rm.z[:, sample] = list(lm.tvalues[:length])
-            rm.est[:, sample] = list(lm.params[:length])
-            rm.se[:, sample] = list(lm.bse[:length])
-    return rm
+
+def _terms(formula, **kwargs):
+    """Extracts all terms from a formula, including those specified in groups
+    and re_formula, which are optionally specified through **kwargs
+    """
+    terms = _split_terms(formula)
+    if 'groups' in kwargs:
+        terms.append(kwargs['groups'])
+    if kwargs.get('re_formula', None) is not None:
+        terms += _split_terms(kwargs['re_formula'])
+    return terms
+
+
+def _colors(colormap, n):
+    cm = plt.colormaps[colormap]
+    return [cm(int(hue)) for hue in np.linspace(0, cm.N, n)]
